
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/python/grapl-common

from setuptools import setup

setup(**{
    'author': 'Grapl, Inc.',
    'author_email': 'grapl-code@graplsecurity.com',
    'classifiers': [
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
    ],
    'description': 'Shared code between Grapl components',
    'license': 'MIT',
    'long_description': """# grapl-common
Shared library between different Grapl python services.

Ideally things that would go in here would be things like:
* serializable and deserializable types (since those cross service boundaries)
* common utilities, i.e. dealing with datetime stuff
""",
    'maintainer': 'Grapl, Inc.',
    'maintainer_email': 'grapl-code@graplsecurity.com',
    'name': 'grapl-common',
    'project_urls': {
        'Documentation': 'https://grapl.readthedocs.io',
        'Source': 'https://github.com/grapl-security/grapl',
        'Tracker': 'https://github.com/grapl-security/grapl/issues',
    },
    'url': 'https://github.com/grapl-security/grapl',
    'version': '0.0.5',
    'package_dir': {
        '': 'src',
    },
    'packages': (
        'grapl_common',
        'grapl_common.debugger',
        'grapl_common.metrics',
    ),
    'namespace_packages': (
    ),
    'package_data': {
        'grapl_common': (
            'py.typed',
        ),
    },
    'install_requires': (
        'mypy_boto3_dynamodb',
        'mypy_boto3_s3',
        'mypy_boto3_secretsmanager',
        'mypy_boto3_sqs',
        'typing_extensions',
    ),
})
